package sun.awt;

import java.awt.Insets;
import java.awt.Adjustable;
import java.awt.event.MouseWheelEvent;
import java.awt.ScrollPane;
import sun.util.logging.PlatformLogger;

public abstract class ScrollPaneWheelScroller
{
    private static final PlatformLogger log;
    
    private ScrollPaneWheelScroller() {
    }
    
    public static void handleWheelScrolling(ScrollPane scrollPane, MouseWheelEvent mouseWheelEvent) {
        if (ScrollPaneWheelScroller.log.isLoggable(PlatformLogger.Level.FINER)) {
            ScrollPaneWheelScroller.log.finer("x = " + mouseWheelEvent.getX() + ", y = " + mouseWheelEvent.getY() + ", src is " + mouseWheelEvent.getSource());
        }
        if (scrollPane != null && mouseWheelEvent.getScrollAmount() != 0) {
            Adjustable adjustableToScroll = getAdjustableToScroll(scrollPane);
            if (adjustableToScroll != null) {
                int incrementFromAdjustable = getIncrementFromAdjustable(adjustableToScroll, mouseWheelEvent);
                if (ScrollPaneWheelScroller.log.isLoggable(PlatformLogger.Level.FINER)) {
                    ScrollPaneWheelScroller.log.finer("increment from adjustable(" + adjustableToScroll.getClass() + ") : " + incrementFromAdjustable);
                }
                scrollAdjustable(adjustableToScroll, incrementFromAdjustable);
            }
        }
    }
    
    public static Adjustable getAdjustableToScroll(ScrollPane scrollPane) {
        int scrollbarDisplayPolicy = scrollPane.getScrollbarDisplayPolicy();
        if (scrollbarDisplayPolicy == 1 || scrollbarDisplayPolicy == 2) {
            if (ScrollPaneWheelScroller.log.isLoggable(PlatformLogger.Level.FINER)) {
                ScrollPaneWheelScroller.log.finer("using vertical scrolling due to scrollbar policy");
            }
            return scrollPane.getVAdjustable();
        }
        else {
            Insets insets = scrollPane.getInsets();
            int vScrollbarWidth = scrollPane.getVScrollbarWidth();
            if (ScrollPaneWheelScroller.log.isLoggable(PlatformLogger.Level.FINER)) {
                ScrollPaneWheelScroller.log.finer("insets: l = " + insets.left + ", r = " + insets.right + ", t = " + insets.top + ", b = " + insets.bottom);
                ScrollPaneWheelScroller.log.finer("vertScrollWidth = " + vScrollbarWidth);
            }
            if (insets.right >= vScrollbarWidth) {
                if (ScrollPaneWheelScroller.log.isLoggable(PlatformLogger.Level.FINER)) {
                    ScrollPaneWheelScroller.log.finer("using vertical scrolling because scrollbar is present");
                }
                return scrollPane.getVAdjustable();
            }
            else if (insets.bottom >= scrollPane.getHScrollbarHeight()) {
                if (ScrollPaneWheelScroller.log.isLoggable(PlatformLogger.Level.FINER)) {
                    ScrollPaneWheelScroller.log.finer("using horiz scrolling because scrollbar is present");
                }
                return scrollPane.getHAdjustable();
            }
            else {
                if (ScrollPaneWheelScroller.log.isLoggable(PlatformLogger.Level.FINER)) {
                    ScrollPaneWheelScroller.log.finer("using NO scrollbar becsause neither is present");
                }
                return null;
            }
        }
    }
    
    public static int getIncrementFromAdjustable(Adjustable adjustable, MouseWheelEvent mouseWheelEvent) {
        if (ScrollPaneWheelScroller.log.isLoggable(PlatformLogger.Level.FINE) && adjustable == null) {
            ScrollPaneWheelScroller.log.fine("Assertion (adj != null) failed");
        }
        int n = 0;
        if (mouseWheelEvent.getScrollType() == 0) {
            n = mouseWheelEvent.getUnitsToScroll() * adjustable.getUnitIncrement();
        }
        else if (mouseWheelEvent.getScrollType() == 1) {
            n = adjustable.getBlockIncrement() * mouseWheelEvent.getWheelRotation();
        }
        return n;
    }
    
    public static void scrollAdjustable(Adjustable adjustable, int i) {
        if (ScrollPaneWheelScroller.log.isLoggable(PlatformLogger.Level.FINE)) {
            if (adjustable == null) {
                ScrollPaneWheelScroller.log.fine("Assertion (adj != null) failed");
            }
            if (i == 0) {
                ScrollPaneWheelScroller.log.fine("Assertion (amount != 0) failed");
            }
        }
        int value = adjustable.getValue();
        int value2 = adjustable.getMaximum() - adjustable.getVisibleAmount();
        if (ScrollPaneWheelScroller.log.isLoggable(PlatformLogger.Level.FINER)) {
            ScrollPaneWheelScroller.log.finer("doScrolling by " + i);
        }
        if (i > 0 && value < value2) {
            if (value + i < value2) {
                adjustable.setValue(value + i);
                return;
            }
            adjustable.setValue(value2);
            return;
        }
        else if (i < 0 && value > adjustable.getMinimum()) {
            if (value + i > adjustable.getMinimum()) {
                adjustable.setValue(value + i);
                return;
            }
            adjustable.setValue(adjustable.getMinimum());
            return;
        }
        else {
            return;
        }
    }
    
    static {
        log = PlatformLogger.getLogger("sun.awt.ScrollPaneWheelScroller");
    }
}
